"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const ramda_1 = require("ramda");
const loader_utils_1 = __importDefault(require("loader-utils"));
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const cwd = process.cwd();
const cache = { fileNameMapping: {} };
function getTSConfigPath() {
    const configPath = typescript_1.default.findConfigFile(cwd, typescript_1.default.sys.fileExists, 'tsconfig.json');
    return configPath;
}
function getTSConfig() {
    const tsconfigPath = getTSConfigPath();
    if (!tsconfigPath) {
        throw new Error("Could not find a valid 'tsconfig.json'.");
    }
    const tsconfig = require(tsconfigPath);
    return tsconfig;
}
const parseConfigHost = {
    fileExists: fs_extra_1.default.existsSync,
    readDirectory: typescript_1.default.sys.readDirectory,
    readFile: function (file) {
        return fs_extra_1.default.readFileSync(file, 'utf8');
    },
    useCaseSensitiveFileNames: true,
};
function getFileNames() {
    const tsconfigPath = getTSConfigPath();
    const tsconfig = getTSConfig();
    if (tsconfigPath) {
        const parsed = typescript_1.default.parseJsonConfigFileContent(tsconfig, parseConfigHost, path_1.default.dirname(tsconfigPath));
        return parsed.fileNames;
    }
    return [];
}
function getTSService(options) {
    if (cache.languageService) {
        return cache.languageService;
    }
    const rootFileNames = getFileNames();
    const files = {};
    rootFileNames.forEach((fileName) => {
        files[fileName] = { version: 0 };
    });
    const servicesHost = {
        getScriptFileNames: () => rootFileNames,
        getScriptVersion: (fileName) => files[fileName] && files[fileName].version.toString(),
        getScriptSnapshot: (fileName) => {
            if (!fs_extra_1.default.existsSync(fileName)) {
                return undefined;
            }
            return typescript_1.default.ScriptSnapshot.fromString(fs_extra_1.default.readFileSync(fileName).toString());
        },
        getCurrentDirectory: () => process.cwd(),
        getCompilationSettings: () => options,
        getDefaultLibFileName: (options) => typescript_1.default.getDefaultLibFilePath(options),
        fileExists: typescript_1.default.sys.fileExists,
        readFile: typescript_1.default.sys.readFile,
        readDirectory: typescript_1.default.sys.readDirectory,
        directoryExists: typescript_1.default.sys.directoryExists,
        getDirectories: typescript_1.default.sys.getDirectories,
    };
    const languageService = typescript_1.default.createLanguageService(servicesHost, typescript_1.default.createDocumentRegistry());
    cache.languageService = languageService;
    return languageService;
}
function emitFile(fileName, languageService, loaderOptions) {
    try {
        const output = languageService.getEmitOutput(fileName);
        if (!output.emitSkipped) {
            output.outputFiles.forEach((o) => {
                if (o.name.endsWith('.d.ts')) {
                    fs_extra_1.default.ensureDirSync(path_1.default.dirname(o.name));
                    fs_extra_1.default.writeFileSync(o.name, o.text);
                    if (loaderOptions.exposes &&
                        !ramda_1.isEmpty(loaderOptions.exposes) &&
                        !ramda_1.isEmpty(loaderOptions.name)) {
                        for (const [key, value] of Object.entries(loaderOptions.exposes)) {
                            if (key && value) {
                                const entryPath = path_1.default.resolve(cwd, value);
                                if (entryPath === fileName) {
                                    const moduleFilename = `${key}.d.ts`;
                                    const modulePath = path_1.default.resolve(cwd, `${loaderOptions.typesOutputDir}/${loaderOptions.name}`);
                                    fs_extra_1.default.writeFileSync(path_1.default.resolve(modulePath, moduleFilename), `export * from './${path_1.default.relative(path_1.default.relative(cwd, modulePath), o.name.replace('.d.ts', ''))}'`);
                                }
                            }
                        }
                    }
                }
            });
        }
    }
    catch (e) {
        console.log(`Skip ${fileName}`);
    }
}
function makeLoader(context, loaderOptions, content) {
    const tsconfig = getTSConfig();
    const languageService = getTSService({
        ...tsconfig,
        declaration: true,
        emitDeclarationOnly: true,
        outDir: path_1.default.resolve(cwd, `${loaderOptions.typesOutputDir}/${loaderOptions.name}/dts`),
    });
    emitFile(context.resourcePath, languageService, loaderOptions);
    return content;
}
function loader(content) {
    const context = this;
    const loaderOptions = loader_utils_1.default.getOptions(context);
    return makeLoader(context, {
        name: loaderOptions.name,
        exposes: loaderOptions.exposes,
        typesOutputDir: loaderOptions.typesOutputDir || '.wp_federation',
    }, content);
}
exports.default = loader;
